#**********************************************************************
# Haar-like 특징을 이용한 Cascade 분류기 (얼굴, 눈 영역)
#**********************************************************************
import cv2 as cv
import argparse                         # 명령행 파싱 모듈

def detectAndDisplay(frame):
    frame_gray = cv.cvtColor(frame, cv.COLOR_BGR2GRAY)  # 그레이 영상으로 칼라정보 변환
    frame_gray = cv.equalizeHist(frame_gray)            # 히스토그램 평활화 - 영상 대비 선명하게
    #-- Detect faces
    faces = face_cascade.detectMultiScale(frame_gray)   # 얼굴영역 탐지
    
    for (x,y,w,h) in faces:
        center = (x + w//2, y + h//2)                   # 얼굴영역 중심 좌표
        frame = cv.ellipse(frame, center, (w//2, h//2), 0, 0, 360, (255, 0, 255), 4)
        faceROI = frame_gray[y:y+h,x:x+w] 
        #-- In each face, detect eyes
        eyes = eyes_cascade.detectMultiScale(faceROI)   # 눈 영역 탐지
        for (x2,y2,w2,h2) in eyes:
            eye_center = (x + x2 + w2//2, y + y2 + h2//2)
            radius = int(round((w2 + h2)*0.25))
            frame = cv.circle(frame, eye_center, radius, (255, 0, 0 ), 4)
    cv.imshow('Capture - Face detection', frame)
    
#=========================================================================================    
parser = argparse.ArgumentParser(description='Code for Cascade Classifier tutorial.')
parser.add_argument('--face_cascade', help='Path to face cascade.', default='./haarcascade_frontalface_alt.xml')
parser.add_argument('--eyes_cascade', help='Path to eyes cascade.', default='./haarcascade_eye_tree_eyeglasses.xml')
parser.add_argument('--camera', help='Camera divide number.', type=int, default=0)
args = parser.parse_args()
# 
face_cascade_name = args.face_cascade
eyes_cascade_name = args.eyes_cascade

# [1] 얼굴과 눈  cascade 분류기 생성
face_cascade = cv.CascadeClassifier()
eyes_cascade = cv.CascadeClassifier()

# [2] 분류기 정보 로딩
if not face_cascade.load(cv.samples.findFile(face_cascade_name)):
    print('--(!)Error loading face cascade')
    exit(0)
if not eyes_cascade.load(cv.samples.findFile(eyes_cascade_name)):
    print('--(!)Error loading eyes cascade')
    exit(0)
    
camera_device = args.camera
# [3] 웹캠으로부터 비디오 정보 입력
cap = cv.VideoCapture(camera_device)
if not cap.isOpened:
    print('--(!)Error opening video capture')
    exit(0)
while True:
    ret, frame = cap.read()
    if frame is None:
        print('--(!) No captured frame -- Break!')
        break
    detectAndDisplay(frame)     # 눈과 얼굴 탐지 함수 호출
    
    if cv.waitKey(10) == 27:
        break